/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2011, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

// --- Level-1 BLAS-like prototypes --------------------------------------------

// --- amax ---

void bli_samax( int n, float*    x, int incx, int* index );
void bli_damax( int n, double*   x, int incx, int* index );
void bli_camax( int n, scomplex* x, int incx, int* index );
void bli_zamax( int n, dcomplex* x, int incx, int* index );

// --- asum ---

void bli_sasum( int n, float*    x, int incx, float*  norm );
void bli_dasum( int n, double*   x, int incx, double* norm );
void bli_casum( int n, scomplex* x, int incx, float*  norm );
void bli_zasum( int n, dcomplex* x, int incx, double* norm );

// --- axpy ---

void bli_saxpy( int n, float*    alpha, float*    x, int incx, float*    y, int incy );
void bli_daxpy( int n, double*   alpha, double*   x, int incx, double*   y, int incy );
void bli_caxpy( int n, scomplex* alpha, scomplex* x, int incx, scomplex* y, int incy );
void bli_zaxpy( int n, dcomplex* alpha, dcomplex* x, int incx, dcomplex* y, int incy );

// --- axpyv ---

void bli_saxpyv( char conj, int n, float*    alpha, float*    x, int incx, float*    y, int incy );
void bli_daxpyv( char conj, int n, double*   alpha, double*   x, int incx, double*   y, int incy );
void bli_caxpyv( char conj, int n, scomplex* alpha, scomplex* x, int incx, scomplex* y, int incy );
void bli_zaxpyv( char conj, int n, dcomplex* alpha, dcomplex* x, int incx, dcomplex* y, int incy );

// --- axpymt ---

void bli_saxpymt( char trans, int m, int n, float*    alpha, float*    a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_daxpymt( char trans, int m, int n, double*   alpha, double*   a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_caxpymt( char trans, int m, int n, scomplex* alpha, scomplex* a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_zaxpymt( char trans, int m, int n, dcomplex* alpha, dcomplex* a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );

// --- axpymrt ---

void bli_saxpymrt( char uplo, char trans, int m, int n, float*    alpha, float*    a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_daxpymrt( char uplo, char trans, int m, int n, double*   alpha, double*   a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_caxpymrt( char uplo, char trans, int m, int n, scomplex* alpha, scomplex* a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_zaxpymrt( char uplo, char trans, int m, int n, dcomplex* alpha, dcomplex* a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );

// --- axpysv ---

void bli_saxpysv( int n, float*    alpha0, float*    alpha1, float*    x, int incx, float*    beta, float*    y, int incy );
void bli_daxpysv( int n, double*   alpha0, double*   alpha1, double*   x, int incx, double*   beta, double*   y, int incy );
void bli_caxpysv( int n, scomplex* alpha0, scomplex* alpha1, scomplex* x, int incx, scomplex* beta, scomplex* y, int incy );
void bli_zaxpysv( int n, dcomplex* alpha0, dcomplex* alpha1, dcomplex* x, int incx, dcomplex* beta, dcomplex* y, int incy );

// --- axpysmt ---

void bli_saxpysmt( char trans, int m, int n, float*    alpha0, float*    alpha1, float*    a, int a_rs, int a_cs, float*    beta, float*    b, int b_rs, int b_cs );
void bli_daxpysmt( char trans, int m, int n, double*   alpha0, double*   alpha1, double*   a, int a_rs, int a_cs, double*   beta, double*   b, int b_rs, int b_cs );
void bli_caxpysmt( char trans, int m, int n, scomplex* alpha0, scomplex* alpha1, scomplex* a, int a_rs, int a_cs, scomplex* beta, scomplex* b, int b_rs, int b_cs );
void bli_zaxpysmt( char trans, int m, int n, dcomplex* alpha0, dcomplex* alpha1, dcomplex* a, int a_rs, int a_cs, dcomplex* beta, dcomplex* b, int b_rs, int b_cs );

// --- conjv ---

void bli_sconjv( int m, float* x, int incx );
void bli_dconjv( int m, double* x, int incx );
void bli_cconjv( int m, scomplex* x, int incx );
void bli_zconjv( int m, dcomplex* x, int incx );

// --- conjm ---

void bli_sconjm( int m, int n, float*    a, int a_rs, int a_cs );
void bli_dconjm( int m, int n, double*   a, int a_rs, int a_cs );
void bli_cconjm( int m, int n, scomplex* a, int a_rs, int a_cs );
void bli_zconjm( int m, int n, dcomplex* a, int a_rs, int a_cs );

// --- conjmr ---

void bli_sconjmr( char uplo, int m, int n, float*    a, int a_rs, int a_cs );
void bli_dconjmr( char uplo, int m, int n, double*   a, int a_rs, int a_cs );
void bli_cconjmr( char uplo, int m, int n, scomplex* a, int a_rs, int a_cs );
void bli_zconjmr( char uplo, int m, int n, dcomplex* a, int a_rs, int a_cs );

// --- copy ---

void bli_scopy( int m, float*    x, int incx, float*    y, int incy );
void bli_dcopy( int m, double*   x, int incx, double*   y, int incy );
void bli_ccopy( int m, scomplex* x, int incx, scomplex* y, int incy );
void bli_zcopy( int m, dcomplex* x, int incx, dcomplex* y, int incy );

// --- copyv ---

void bli_icopyv( char conj, int m, int*      x, int incx, int*      y, int incy );
void bli_scopyv( char conj, int m, float*    x, int incx, float*    y, int incy );
void bli_dcopyv( char conj, int m, double*   x, int incx, double*   y, int incy );
void bli_ccopyv( char conj, int m, scomplex* x, int incx, scomplex* y, int incy );
void bli_zcopyv( char conj, int m, dcomplex* x, int incx, dcomplex* y, int incy );

void bli_sdcopyv( char conj, int m, float*    x, int incx, double*   y, int incy );
void bli_dscopyv( char conj, int m, double*   x, int incx, float*    y, int incy );
void bli_sccopyv( char conj, int m, float*    x, int incx, scomplex* y, int incy );
void bli_cscopyv( char conj, int m, scomplex* x, int incx, float*    y, int incy );
void bli_szcopyv( char conj, int m, float*    x, int incx, dcomplex* y, int incy );
void bli_zscopyv( char conj, int m, dcomplex* x, int incx, float*    y, int incy );
void bli_dccopyv( char conj, int m, double*   x, int incx, scomplex* y, int incy );
void bli_cdcopyv( char conj, int m, scomplex* x, int incx, double*   y, int incy );
void bli_dzcopyv( char conj, int m, double*   x, int incx, dcomplex* y, int incy );
void bli_zdcopyv( char conj, int m, dcomplex* x, int incx, double*   y, int incy );
void bli_czcopyv( char conj, int m, scomplex* x, int incx, dcomplex* y, int incy );
void bli_zccopyv( char conj, int m, dcomplex* x, int incx, scomplex* y, int incy );

// --- copymr ---

void bli_scopymr( char uplo, int m, int n, float*    a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_dcopymr( char uplo, int m, int n, double*   a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_ccopymr( char uplo, int m, int n, scomplex* a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_zcopymr( char uplo, int m, int n, dcomplex* a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );

void bli_sscopymr( char uplo, int m, int n, float*    a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_sdcopymr( char uplo, int m, int n, float*    a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_dscopymr( char uplo, int m, int n, double*   a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_sccopymr( char uplo, int m, int n, float*    a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_cscopymr( char uplo, int m, int n, scomplex* a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_szcopymr( char uplo, int m, int n, float*    a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );
void bli_zscopymr( char uplo, int m, int n, dcomplex* a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_ddcopymr( char uplo, int m, int n, double*   a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_dccopymr( char uplo, int m, int n, double*   a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_cdcopymr( char uplo, int m, int n, scomplex* a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_dzcopymr( char uplo, int m, int n, double*   a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );
void bli_zdcopymr( char uplo, int m, int n, dcomplex* a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_cccopymr( char uplo, int m, int n, scomplex* a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_czcopymr( char uplo, int m, int n, scomplex* a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );
void bli_zccopymr( char uplo, int m, int n, dcomplex* a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_zzcopymr( char uplo, int m, int n, dcomplex* a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );

// --- copymrt ---

void bli_scopymrt( char uplo, char trans, int m, int n, float*    a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_dcopymrt( char uplo, char trans, int m, int n, double*   a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_ccopymrt( char uplo, char trans, int m, int n, scomplex* a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_zcopymrt( char uplo, char trans, int m, int n, dcomplex* a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );

void bli_sscopymrt( char uplo, char trans, int m, int n, float*    a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_sdcopymrt( char uplo, char trans, int m, int n, float*    a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_sccopymrt( char uplo, char trans, int m, int n, float*    a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_szcopymrt( char uplo, char trans, int m, int n, float*    a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );
void bli_dscopymrt( char uplo, char trans, int m, int n, double*   a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_ddcopymrt( char uplo, char trans, int m, int n, double*   a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_dccopymrt( char uplo, char trans, int m, int n, double*   a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_dzcopymrt( char uplo, char trans, int m, int n, double*   a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );
void bli_cscopymrt( char uplo, char trans, int m, int n, scomplex* a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_cdcopymrt( char uplo, char trans, int m, int n, scomplex* a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_cccopymrt( char uplo, char trans, int m, int n, scomplex* a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_czcopymrt( char uplo, char trans, int m, int n, scomplex* a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );
void bli_zscopymrt( char uplo, char trans, int m, int n, dcomplex* a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_zdcopymrt( char uplo, char trans, int m, int n, dcomplex* a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_zccopymrt( char uplo, char trans, int m, int n, dcomplex* a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_zzcopymrt( char uplo, char trans, int m, int n, dcomplex* a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );

// --- copymt ---

void bli_icopymt( char trans, int m, int n, int*      a, int a_rs, int a_cs, int*      b, int b_rs, int b_cs );
void bli_scopymt( char trans, int m, int n, float*    a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_dcopymt( char trans, int m, int n, double*   a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_ccopymt( char trans, int m, int n, scomplex* a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_zcopymt( char trans, int m, int n, dcomplex* a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );

void bli_sscopymt( char trans, int m, int n, float*    a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_sdcopymt( char trans, int m, int n, float*    a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_dscopymt( char trans, int m, int n, double*   a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_sccopymt( char trans, int m, int n, float*    a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_cscopymt( char trans, int m, int n, scomplex* a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_szcopymt( char trans, int m, int n, float*    a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );
void bli_zscopymt( char trans, int m, int n, dcomplex* a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_ddcopymt( char trans, int m, int n, double*   a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_dccopymt( char trans, int m, int n, double*   a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_cdcopymt( char trans, int m, int n, scomplex* a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_dzcopymt( char trans, int m, int n, double*   a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );
void bli_zdcopymt( char trans, int m, int n, dcomplex* a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_cccopymt( char trans, int m, int n, scomplex* a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_czcopymt( char trans, int m, int n, scomplex* a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );
void bli_zccopymt( char trans, int m, int n, dcomplex* a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_zzcopymt( char trans, int m, int n, dcomplex* a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );

// --- dot ---

void bli_cdot_in( char conj, int n, scomplex* x, int incx, scomplex* y, int incy, scomplex* rho );
void bli_zdot_in( char conj, int n, dcomplex* x, int incx, dcomplex* y, int incy, dcomplex* rho );

void bli_sdot( char conj, int n, float*    x, int incx, float*    y, int incy, float*    rho );
void bli_ddot( char conj, int n, double*   x, int incx, double*   y, int incy, double*   rho );
void bli_cdot( char conj, int n, scomplex* x, int incx, scomplex* y, int incy, scomplex* rho );
void bli_zdot( char conj, int n, dcomplex* x, int incx, dcomplex* y, int incy, dcomplex* rho );

// --- dots ---

void bli_sdots( char conj, int n, float*    alpha, float*    x, int incx, float*    y, int incy, float*    beta, float*    rho );
void bli_ddots( char conj, int n, double*   alpha, double*   x, int incx, double*   y, int incy, double*   beta, double*   rho );
void bli_cdots( char conj, int n, scomplex* alpha, scomplex* x, int incx, scomplex* y, int incy, scomplex* beta, scomplex* rho );
void bli_zdots( char conj, int n, dcomplex* alpha, dcomplex* x, int incx, dcomplex* y, int incy, dcomplex* beta, dcomplex* rho );

// --- dot2s ---

void bli_sdot2s( char conj, int n, float*    alpha, float*    x, int incx, float*    y, int incy, float*    beta, float*    rho );
void bli_ddot2s( char conj, int n, double*   alpha, double*   x, int incx, double*   y, int incy, double*   beta, double*   rho );
void bli_cdot2s( char conj, int n, scomplex* alpha, scomplex* x, int incx, scomplex* y, int incy, scomplex* beta, scomplex* rho );
void bli_zdot2s( char conj, int n, dcomplex* alpha, dcomplex* x, int incx, dcomplex* y, int incy, dcomplex* beta, dcomplex* rho );

// --- fnorm ---

void bli_sfnorm( int m, int n, float*    a, int a_rs, int a_cs, float*  norm );
void bli_dfnorm( int m, int n, double*   a, int a_rs, int a_cs, double* norm );
void bli_cfnorm( int m, int n, scomplex* a, int a_rs, int a_cs, float*  norm );
void bli_zfnorm( int m, int n, dcomplex* a, int a_rs, int a_cs, double* norm );

// --- invscalv ---

void bli_sinvscalv(  char conj, int n, float*    alpha, float*    x, int incx );
void bli_dinvscalv(  char conj, int n, double*   alpha, double*   x, int incx );
void bli_csinvscalv( char conj, int n, float*    alpha, scomplex* x, int incx );
void bli_cinvscalv(  char conj, int n, scomplex* alpha, scomplex* x, int incx );
void bli_zdinvscalv( char conj, int n, double*   alpha, dcomplex* x, int incx );
void bli_zinvscalv(  char conj, int n, dcomplex* alpha, dcomplex* x, int incx );

// --- invscalm ---

void bli_sinvscalm(  char conj, int m, int n, float*    alpha, float*    a, int a_rs, int a_cs );
void bli_dinvscalm(  char conj, int m, int n, double*   alpha, double*   a, int a_rs, int a_cs );
void bli_csinvscalm( char conj, int m, int n, float*    alpha, scomplex* a, int a_rs, int a_cs );
void bli_cinvscalm(  char conj, int m, int n, scomplex* alpha, scomplex* a, int a_rs, int a_cs );
void bli_zdinvscalm( char conj, int m, int n, double*   alpha, dcomplex* a, int a_rs, int a_cs );
void bli_zinvscalm(  char conj, int m, int n, dcomplex* alpha, dcomplex* a, int a_rs, int a_cs );

// --- nrm2 ---

void bli_snrm2( int n, float*    x, int incx, float*  norm );
void bli_dnrm2( int n, double*   x, int incx, double* norm );
void bli_cnrm2( int n, scomplex* x, int incx, float*  norm );
void bli_znrm2( int n, dcomplex* x, int incx, double* norm );

// --- scal ---

void bli_sscal(  int n, float*    alpha, float*    x, int incx );
void bli_dscal(  int n, double*   alpha, double*   x, int incx );
void bli_csscal( int n, float*    alpha, scomplex* x, int incx );
void bli_cscal(  int n, scomplex* alpha, scomplex* x, int incx );
void bli_zdscal( int n, double*   alpha, dcomplex* x, int incx );
void bli_zscal(  int n, dcomplex* alpha, dcomplex* x, int incx );

// --- scalv ---

void bli_sscalv(  char conj, int n, float*    alpha, float*    x, int incx );
void bli_dscalv(  char conj, int n, double*   alpha, double*   x, int incx );
void bli_csscalv( char conj, int n, float*    alpha, scomplex* x, int incx );
void bli_cscalv(  char conj, int n, scomplex* alpha, scomplex* x, int incx );
void bli_zdscalv( char conj, int n, double*   alpha, dcomplex* x, int incx );
void bli_zscalv(  char conj, int n, dcomplex* alpha, dcomplex* x, int incx );

// --- scalm ---

void bli_sscalm(  char conj, int m, int n, float*    alpha, float*    a, int a_rs, int a_cs );
void bli_dscalm(  char conj, int m, int n, double*   alpha, double*   a, int a_rs, int a_cs );
void bli_csscalm( char conj, int m, int n, float*    alpha, scomplex* a, int a_rs, int a_cs );
void bli_cscalm(  char conj, int m, int n, scomplex* alpha, scomplex* a, int a_rs, int a_cs );
void bli_zdscalm( char conj, int m, int n, double*   alpha, dcomplex* a, int a_rs, int a_cs );
void bli_zscalm(  char conj, int m, int n, dcomplex* alpha, dcomplex* a, int a_rs, int a_cs );

// --- scalmr ---

void bli_sscalmr(  char uplo, int m, int n, float*    alpha, float*    a, int a_rs, int a_cs );
void bli_dscalmr(  char uplo, int m, int n, double*   alpha, double*   a, int a_rs, int a_cs );
void bli_csscalmr( char uplo, int m, int n, float*    alpha, scomplex* a, int a_rs, int a_cs );
void bli_cscalmr(  char uplo, int m, int n, scomplex* alpha, scomplex* a, int a_rs, int a_cs );
void bli_zdscalmr( char uplo, int m, int n, double*   alpha, dcomplex* a, int a_rs, int a_cs );
void bli_zscalmr(  char uplo, int m, int n, dcomplex* alpha, dcomplex* a, int a_rs, int a_cs );

// --- swap ---

void bli_sswap( int n, float*    x, int incx, float*    y, int incy );
void bli_dswap( int n, double*   x, int incx, double*   y, int incy );
void bli_cswap( int n, scomplex* x, int incx, scomplex* y, int incy );
void bli_zswap( int n, dcomplex* x, int incx, dcomplex* y, int incy );

// --- swapv ---

void bli_sswapv( int n, float*    x, int incx, float*    y, int incy );
void bli_dswapv( int n, double*   x, int incx, double*   y, int incy );
void bli_cswapv( int n, scomplex* x, int incx, scomplex* y, int incy );
void bli_zswapv( int n, dcomplex* x, int incx, dcomplex* y, int incy );

// --- swapmt ---

void bli_sswapmt( char trans, int m, int n, float*    a, int a_rs, int a_cs, float*    b, int b_rs, int b_cs );
void bli_dswapmt( char trans, int m, int n, double*   a, int a_rs, int a_cs, double*   b, int b_rs, int b_cs );
void bli_cswapmt( char trans, int m, int n, scomplex* a, int a_rs, int a_cs, scomplex* b, int b_rs, int b_cs );
void bli_zswapmt( char trans, int m, int n, dcomplex* a, int a_rs, int a_cs, dcomplex* b, int b_rs, int b_cs );

