/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2011, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

// --- Level-2 BLAS-like prototypes --------------------------------------------

// --- gemv ---

void bli_sgemv( char transa, char conjx, int m, int n, float*    alpha, float*    a, int a_rs, int a_cs, float*    x, int incx, float*    beta, float*    y, int incy );
void bli_dgemv( char transa, char conjx, int m, int n, double*   alpha, double*   a, int a_rs, int a_cs, double*   x, int incx, double*   beta, double*   y, int incy );
void bli_cgemv( char transa, char conjx, int m, int n, scomplex* alpha, scomplex* a, int a_rs, int a_cs, scomplex* x, int incx, scomplex* beta, scomplex* y, int incy );
void bli_zgemv( char transa, char conjx, int m, int n, dcomplex* alpha, dcomplex* a, int a_rs, int a_cs, dcomplex* x, int incx, dcomplex* beta, dcomplex* y, int incy );

void bli_sgemv_blas( char transa, int m, int n, float*    alpha, float*    a, int lda, float*    x, int incx, float*    beta, float*    y, int incy );
void bli_dgemv_blas( char transa, int m, int n, double*   alpha, double*   a, int lda, double*   x, int incx, double*   beta, double*   y, int incy );
void bli_cgemv_blas( char transa, int m, int n, scomplex* alpha, scomplex* a, int lda, scomplex* x, int incx, scomplex* beta, scomplex* y, int incy );
void bli_zgemv_blas( char transa, int m, int n, dcomplex* alpha, dcomplex* a, int lda, dcomplex* x, int incx, dcomplex* beta, dcomplex* y, int incy );

// --- ger ---

void bli_sger( char conjx, char conjy, int m, int n, float*    alpha, float*    x, int incx, float*    y, int incy, float*    a, int a_rs, int a_cs );
void bli_dger( char conjx, char conjy, int m, int n, double*   alpha, double*   x, int incx, double*   y, int incy, double*   a, int a_rs, int a_cs );
void bli_cger( char conjx, char conjy, int m, int n, scomplex* alpha, scomplex* x, int incx, scomplex* y, int incy, scomplex* a, int a_rs, int a_cs );
void bli_zger( char conjx, char conjy, int m, int n, dcomplex* alpha, dcomplex* x, int incx, dcomplex* y, int incy, dcomplex* a, int a_rs, int a_cs );

void bli_sger_blas(  int m, int n, float*    alpha, float*    x, int incx, float*    y, int incy, float*    a, int lda );
void bli_dger_blas(  int m, int n, double*   alpha, double*   x, int incx, double*   y, int incy, double*   a, int lda );
void bli_cgerc_blas( int m, int n, scomplex* alpha, scomplex* x, int incx, scomplex* y, int incy, scomplex* a, int lda );
void bli_cgeru_blas( int m, int n, scomplex* alpha, scomplex* x, int incx, scomplex* y, int incy, scomplex* a, int lda );
void bli_zgerc_blas( int m, int n, dcomplex* alpha, dcomplex* x, int incx, dcomplex* y, int incy, dcomplex* a, int lda );
void bli_zgeru_blas( int m, int n, dcomplex* alpha, dcomplex* x, int incx, dcomplex* y, int incy, dcomplex* a, int lda );

// --- hemv ---

void bli_shemv( char uplo, char conj, int m, float*    alpha, float*    a, int a_rs, int a_cs, float*    x, int incx, float*    beta, float*    y, int incy );
void bli_dhemv( char uplo, char conj, int m, double*   alpha, double*   a, int a_rs, int a_cs, double*   x, int incx, double*   beta, double*   y, int incy );
void bli_chemv( char uplo, char conj, int m, scomplex* alpha, scomplex* a, int a_rs, int a_cs, scomplex* x, int incx, scomplex* beta, scomplex* y, int incy );
void bli_zhemv( char uplo, char conj, int m, dcomplex* alpha, dcomplex* a, int a_rs, int a_cs, dcomplex* x, int incx, dcomplex* beta, dcomplex* y, int incy );

void bli_chemv_blas( char uplo, int m, scomplex* alpha, scomplex* a, int lda, scomplex* x, int incx, scomplex* beta, scomplex* y, int incy );
void bli_zhemv_blas( char uplo, int m, dcomplex* alpha, dcomplex* a, int lda, dcomplex* x, int incx, dcomplex* beta, dcomplex* y, int incy );

// --- her ---

void bli_sher( char uplo, char conj, int m, float*  alpha, float*    x, int incx, float*    a, int a_rs, int a_cs );
void bli_dher( char uplo, char conj, int m, double* alpha, double*   x, int incx, double*   a, int a_rs, int a_cs );
void bli_cher( char uplo, char conj, int m, float*  alpha, scomplex* x, int incx, scomplex* a, int a_rs, int a_cs );
void bli_zher( char uplo, char conj, int m, double* alpha, dcomplex* x, int incx, dcomplex* a, int a_rs, int a_cs );

void bli_cher_blas( char uplo, int m, float*  alpha, scomplex* x, int incx, scomplex* a, int lda );
void bli_zher_blas( char uplo, int m, double* alpha, dcomplex* x, int incx, dcomplex* a, int lda );

// --- her2 ---

void bli_sher2( char uplo, char conj, int m, float*    alpha, float*    x, int incx, float*    y, int incy, float*    a, int a_rs, int a_cs );
void bli_dher2( char uplo, char conj, int m, double*   alpha, double*   x, int incx, double*   y, int incy, double*   a, int a_rs, int a_cs );
void bli_cher2( char uplo, char conj, int m, scomplex* alpha, scomplex* x, int incx, scomplex* y, int incy, scomplex* a, int a_rs, int a_cs );
void bli_zher2( char uplo, char conj, int m, dcomplex* alpha, dcomplex* x, int incx, dcomplex* y, int incy, dcomplex* a, int a_rs, int a_cs );

void bli_cher2_blas( char uplo, int m, scomplex* alpha, scomplex* x, int incx, scomplex* y, int incy, scomplex* a, int lda );
void bli_zher2_blas( char uplo, int m, dcomplex* alpha, dcomplex* x, int incx, dcomplex* y, int incy, dcomplex* a, int lda );

// --- symv ---

void bli_ssymv( char uplo, int m, float*    alpha, float*    a, int a_rs, int a_cs, float*    x, int incx, float*    beta, float*    y, int incy );
void bli_dsymv( char uplo, int m, double*   alpha, double*   a, int a_rs, int a_cs, double*   x, int incx, double*   beta, double*   y, int incy );
void bli_csymv( char uplo, int m, scomplex* alpha, scomplex* a, int a_rs, int a_cs, scomplex* x, int incx, scomplex* beta, scomplex* y, int incy );
void bli_zsymv( char uplo, int m, dcomplex* alpha, dcomplex* a, int a_rs, int a_cs, dcomplex* x, int incx, dcomplex* beta, dcomplex* y, int incy );

void bli_ssymv_blas( char uplo, int m, float*    alpha, float*    a, int lda, float*    x, int incx, float*    beta, float*    y, int incy );
void bli_dsymv_blas( char uplo, int m, double*   alpha, double*   a, int lda, double*   x, int incx, double*   beta, double*   y, int incy );
void bli_csymv_blas( char uplo, int m, scomplex* alpha, scomplex* a, int lda, scomplex* x, int incx, scomplex* beta, scomplex* y, int incy );
void bli_zsymv_blas( char uplo, int m, dcomplex* alpha, dcomplex* a, int lda, dcomplex* x, int incx, dcomplex* beta, dcomplex* y, int incy );

// --- syr ---

void bli_ssyr( char uplo, int m, float*    alpha, float*    x, int incx, float*    a, int a_rs, int a_cs );
void bli_dsyr( char uplo, int m, double*   alpha, double*   x, int incx, double*   a, int a_rs, int a_cs );
void bli_csyr( char uplo, int m, scomplex* alpha, scomplex* x, int incx, scomplex* a, int a_rs, int a_cs );
void bli_zsyr( char uplo, int m, dcomplex* alpha, dcomplex* x, int incx, dcomplex* a, int a_rs, int a_cs );

void bli_ssyr_blas( char uplo, int m, float*    alpha, float*    x, int incx, float*    a, int lda );
void bli_dsyr_blas( char uplo, int m, double*   alpha, double*   x, int incx, double*   a, int lda );
void bli_csyr_blas( char uplo, int m, scomplex* alpha, scomplex* x, int incx, scomplex* a, int lda );
void bli_zsyr_blas( char uplo, int m, dcomplex* alpha, dcomplex* x, int incx, dcomplex* a, int lda );

// --- syr2 ---

void bli_ssyr2( char uplo, int m, float*    alpha, float*    x, int incx, float*    y, int incy, float*    a, int a_rs, int a_cs );
void bli_dsyr2( char uplo, int m, double*   alpha, double*   x, int incx, double*   y, int incy, double*   a, int a_rs, int a_cs );
void bli_csyr2( char uplo, int m, scomplex* alpha, scomplex* x, int incx, scomplex* y, int incy, scomplex* a, int a_rs, int a_cs );
void bli_zsyr2( char uplo, int m, dcomplex* alpha, dcomplex* x, int incx, dcomplex* y, int incy, dcomplex* a, int a_rs, int a_cs );

void bli_ssyr2_blas( char uplo, int m, float*    alpha, float*    x, int incx, float*    y, int incy, float*    a, int lda );
void bli_dsyr2_blas( char uplo, int m, double*   alpha, double*   x, int incx, double*   y, int incy, double*   a, int lda );
void bli_csyr2_blas( char uplo, int m, scomplex* alpha, scomplex* x, int incx, scomplex* y, int incy, scomplex* a, int lda );
void bli_zsyr2_blas( char uplo, int m, dcomplex* alpha, dcomplex* x, int incx, dcomplex* y, int incy, dcomplex* a, int lda );

// --- trmv ---

void bli_strmv( char uplo, char trans, char diag, int m, float*    a, int a_rs, int a_cs, float*    x, int incx );
void bli_dtrmv( char uplo, char trans, char diag, int m, double*   a, int a_rs, int a_cs, double*   x, int incx );
void bli_ctrmv( char uplo, char trans, char diag, int m, scomplex* a, int a_rs, int a_cs, scomplex* x, int incx );
void bli_ztrmv( char uplo, char trans, char diag, int m, dcomplex* a, int a_rs, int a_cs, dcomplex* x, int incx );

void bli_strmv_blas( char uplo, char trans, char diag, int m, float*    a, int lda, float*    x, int incx );
void bli_dtrmv_blas( char uplo, char trans, char diag, int m, double*   a, int lda, double*   x, int incx );
void bli_ctrmv_blas( char uplo, char trans, char diag, int m, scomplex* a, int lda, scomplex* x, int incx );
void bli_ztrmv_blas( char uplo, char trans, char diag, int m, dcomplex* a, int lda, dcomplex* x, int incx );

// --- trsv ---

void bli_strsv( char uplo, char trans, char diag, int m, float*    a, int a_rs, int a_cs, float*    x, int incx );
void bli_dtrsv( char uplo, char trans, char diag, int m, double*   a, int a_rs, int a_cs, double*   x, int incx );
void bli_ctrsv( char uplo, char trans, char diag, int m, scomplex* a, int a_rs, int a_cs, scomplex* x, int incx );
void bli_ztrsv( char uplo, char trans, char diag, int m, dcomplex* a, int a_rs, int a_cs, dcomplex* x, int incx );

void bli_strsv_blas( char uplo, char trans, char diag, int m, float*    a, int lda, float*    x, int incx );
void bli_dtrsv_blas( char uplo, char trans, char diag, int m, double*   a, int lda, double*   x, int incx );
void bli_ctrsv_blas( char uplo, char trans, char diag, int m, scomplex* a, int lda, scomplex* x, int incx );
void bli_ztrsv_blas( char uplo, char trans, char diag, int m, dcomplex* a, int lda, dcomplex* x, int incx );

// --- trmvsx ---

void bli_strmvsx( char uplo, char trans, char diag, int m, float* alpha, float* a, int a_rs, int a_cs, float* x, int incx, float* beta, float* y, int incy );
void bli_dtrmvsx( char uplo, char trans, char diag, int m, double* alpha, double* a, int a_rs, int a_cs, double* x, int incx, double* beta, double* y, int incy );
void bli_ctrmvsx( char uplo, char trans, char diag, int m, scomplex* alpha, scomplex* a, int a_rs, int a_cs, scomplex* x, int incx, scomplex* beta, scomplex* y, int incy );
void bli_ztrmvsx( char uplo, char trans, char diag, int m, dcomplex* alpha, dcomplex* a, int a_rs, int a_cs, dcomplex* x, int incx, dcomplex* beta, dcomplex* y, int incy );

// --- trsvsx ---

void bli_strsvsx( char uplo, char trans, char diag, int m, float* alpha, float* a, int a_rs, int a_cs, float* x, int incx, float* beta, float* y, int incy );
void bli_dtrsvsx( char uplo, char trans, char diag, int m, double* alpha, double* a, int a_rs, int a_cs, double* x, int incx, double* beta, double* y, int incy );
void bli_ctrsvsx( char uplo, char trans, char diag, int m, scomplex* alpha, scomplex* a, int a_rs, int a_cs, scomplex* x, int incx, scomplex* beta, scomplex* y, int incy );
void bli_ztrsvsx( char uplo, char trans, char diag, int m, dcomplex* alpha, dcomplex* a, int a_rs, int a_cs, dcomplex* x, int incx, dcomplex* beta, dcomplex* y, int incy );

